module Estatic
  class Category < Resource
    attr_accessor :subcategories, :products

    def initialize(name, subcategories = [])
      super(name)
      @subcategories = subcategories.map { |subcategory| Subcategory.new(subcategory, name) }
      @products = get_products
    end

    def has_subcategories?
      subcategories.any?
    end

    def get_products
      if has_subcategories?
        subcategories.map(&:products).flatten
      else
        products = CSVParser.parse(name)
        products.map { |attributes| OpenStruct.new product(attributes) }
      end
    end

    def directory
      File.join(Estatic.configuration.estatic_site_path, name)
    end

    def root_path
      '../'.freeze
    end
  end
end
