module Estatic
  class Resource
    attr_accessor :name, :locals

    def initialize(name)
      @name = name
      @locals = get_locals
    end

    def get_locals
      locals = Estatic.configuration.locals.detect { |e| e.keys.first == name }&.values&.first || []
      locals.map { |local| [local.keys.first, local.values.first] }.to_h
    end

    def products_in_chunks
      @chunks ||= products.each_slice(Estatic.configuration.chunk).to_a
    end

    def total_chunks
      products_in_chunks.size
    end

    def directory
      File.join(Estatic.configuration.estatic_site_path, name)
    end

    def proper_name
      name.gsub('-', ' ').split.map(&:capitalize).join(' ')
    end

    private

    def product(attributes)
      product = attributes.map do |key, value|
        if key.start_with? 'json_'
          [key.sub('json_', ''), JSON.parse(value)]
        else
          [key, value]
        end
      end
      product.to_h
    end
  end
end
